import NodeType from './NodeType';
import Document from './Document';
import EventTarget from '../event/EventTarget';
import MutationObserverListener from '../mutation-observer/MutationListener';
/**
 * Node
 */
export default class Node extends EventTarget {
    static ownerDocument: Document;
    ownerDocument: Document;
    parentNode: Node;
    readonly nodeType: NodeType;
    readonly childNodes: Node[];
    protected _isConnected: boolean;
    protected observers: MutationObserverListener[];
    /**
     * Constructor.
     */
    constructor();
    /**
     * "true" if connected to DOM.
     *
     * @return {boolean} "true" if connected.
     */
    /**
    * Sets the connected state.
    *
    * @param {boolean} isConnected "true" if connected.
    */
    isConnected: boolean;
    /**
     * Node value.
     *
     * @return {string} Node value.
     */
    readonly nodeValue: string;
    /**
     * Node name.
     *
     * @return {string} Node name.
     */
    readonly nodeName: string;
    /**
     * Previous sibling.
     *
     * @return {Node} Node.
     */
    readonly previousSibling: Node;
    /**
     * Next sibling.
     *
     * @return {Node} Node.
     */
    readonly nextSibling: Node;
    /**
     * First child.
     *
     * @return {Node} Node.
     */
    readonly firstChild: Node;
    /**
     * Last child.
     *
     * @return {Node} Node.
     */
    readonly lastChild: Node;
    /**
     * Connected callback.
     */
    connectedCallback?(): void;
    /**
     * Disconnected callback.
     */
    disconnectedCallback?(): void;
    /**
     * Returns "true" if the node has attributes.
     *
     * @return {boolean} "true" if the node has attributes.
     */
    hasAttributes(): boolean;
    /**
     * Returns a unique ID.
     *
     * @return {string} ID.
     */
    getUniqueID(): string;
    /**
     * Clones a node.
     *
     * @return {Node} Cloned node.
     */
    cloneNode(): Node;
    /**
     * Append a child node to childNodes.
     *
     * @param  {Node} node Node to append.
     * @return {Node} Appended node.
     */
    appendChild(node: Node): Node;
    /**
     * Remove Child element from childNodes array.
     *
     * @param {Node} node Node to remove
     */
    removeChild(node: Node): void;
    /**
     * Inserts a node before another.
     *
     * @param {Node} newNode Node to insert.
     * @param {Node} referenceNode Node to insert before.
     * @return {Node} Inserted node.
     */
    insertBefore(newNode: Node, referenceNode: Node): Node;
    /**
     * Replaces a node with another.
     *
     * @param {Node} newChild New child.
     * @param {Node} oldChild Old child.
     * @return {Node} Replaced node.
     */
    replaceChild(newChild: Node, oldChild: Node): Node;
    /**
     * Observeres the node.
     * Used by MutationObserver, but it is not part of the HTML standard.
     *
     * @param {MutationObserverListener} listener Listener.
     */
    observe(listener: MutationObserverListener): void;
    /**
     * Stops observing the node.
     * Used by MutationObserver, but it is not part of the HTML standard.
     *
     * @param {MutationObserverListener} listener Listener.
     */
    unobserve(listener: MutationObserverListener): void;
}
